// msgqueue.cpp : Implementation of Cmsgqueue
#include "stdafx.h"
#include "Asp1801.h"
#include "msgqueue.h"

#import "mqoa.dll" no_namespace

/////////////////////////////////////////////////////////////////////////////
// Cmsgqueue


STDMETHODIMP Cmsgqueue::newPublicQueue(BSTR bstrQueue, BSTR bstrLabel, BSTR* pbstrMessage)
{
    CComBSTR bstrMessage;
    bstrMessage = "Queue Created";

    IMSMQQueueInfoPtr qinfo("MSMQ.MSMQQueueInfo");
    
    qinfo->PathName = _bstr_t(".\\") + _bstr_t(bstrQueue);    
    qinfo->Label = bstrLabel;
    try
    {
      qinfo->Create();
    }
    catch (_com_error comerr)
    {
      HRESULT hr = comerr.Error();
      if (hr == MQ_ERROR_QUEUE_EXISTS)
      {
        bstrMessage = "Queue already exists";
      }
      else
      {
        bstrMessage = "Could not create queue";
      }
    }

    *pbstrMessage = bstrMessage.Detach();
 
	return S_OK;
}

STDMETHODIMP Cmsgqueue::newPrivateQueue(BSTR bstrQueue, BSTR bstrLabel, BSTR *pbstrMessage)
{
    CComBSTR bstrMessage;
    bstrMessage = "Queue Created";

    IMSMQQueueInfoPtr qinfo("MSMQ.MSMQQueueInfo");
    
    qinfo->PathName = _bstr_t(".\\PRIVATE$\\") + _bstr_t(bstrQueue);    
    qinfo->Label = bstrLabel;
    try
    {
      qinfo->Create();
    }
    catch (_com_error comerr)
    {
      HRESULT hr = comerr.Error();
      if (hr == MQ_ERROR_QUEUE_EXISTS)
      {
        bstrMessage = "Queue already exists";
      }
      else
      {
        bstrMessage = "Could not create queue";
      }
    }

    *pbstrMessage = bstrMessage.Detach();
 
	return S_OK;
}

STDMETHODIMP Cmsgqueue::removeQueue(BSTR bstrQueue, int iType, BSTR *pbstrMessage)
{
    CComBSTR bstrMessage;
    bstrMessage = "Queue Removed";

    IMSMQQueueInfoPtr qinfo("MSMQ.MSMQQueueInfo");
    
    if (iType == 1)
       qinfo->PathName = _bstr_t(".\\PRIVATE$\\") + _bstr_t(bstrQueue); 
    else
       qinfo->PathName = _bstr_t(".\\") + _bstr_t(bstrQueue); 

    // obtain reference to queue
    try
    {
      qinfo->Create();
    }
    catch (_com_error comerr)
    {
      HRESULT hr = comerr.Error();
      if (hr == MQ_ERROR_QUEUE_EXISTS)
      {
        // disregard
      }
      else
      {
        bstrMessage = "Could not create queue";
        *pbstrMessage = bstrMessage.Detach();
        return hr;
      }
    }

    // delete queue
    try {
        qinfo->Delete();
    }

    catch (_com_error comerr)
    {
        bstrMessage = "Could not remove queue";
    }


    *pbstrMessage = bstrMessage.Detach();
	return S_OK;
}

STDMETHODIMP Cmsgqueue::findQueues(BSTR bstrLabel)
{
   HRESULT hr = S_OK;

   // **** create Response object **** //

   CComPtr<IObjectContext> spObjectContext;
   CComPtr<IResponse> piResponse;
   CComBSTR bstrObj;
   CComVariant vt;
   CComPtr<IGetContextProperties> pProps; //Context Properties

   IDispatch* piDispatch = NULL;

   // get ObjectContext
	hr = CoGetObjectContext(IID_IObjectContext, 
                                    (void **)&spObjectContext);
   if (FAILED(hr))
      return hr;
   
   // get Context Properties
   hr = spObjectContext->QueryInterface( IID_IGetContextProperties, 
               (void**)&pProps );
   if (FAILED(hr)) 
       return hr;

   // get Response property
   bstrObj = "Response";
   hr = pProps->GetProperty( bstrObj, &vt ) ;

   if (FAILED(hr))
       return hr;

   piDispatch = vt. pdispVal;
   hr = piDispatch->QueryInterface( IID_IResponse, 
                     (void**)&piResponse );

   // **** MSMQ Functionality **** //
   
  IMSMQQueryPtr query("MSMQ.MSMQQuery");
  IMSMQQueueInfosPtr qinfos;
  IMSMQQueueInfoPtr qinfo;
  
  _variant_t vtLabel;
  
  vtLabel = bstrLabel;
  
  // lookup based on label
  qinfos = query->LookupQueue(&vtMissing, &vtMissing, &vtLabel);
  
  // get first
  qinfo = qinfos->Next();

  _bstr_t bstrFormatName;
  _bstr_t bstrPathName;

  // if no queues found
  if (qinfo == NULL) {
     piResponse->Write (_variant_t("no queues matching the label"));
     return S_OK;
  }

  // while queues
  while (qinfo != NULL)
  {
    bstrFormatName = qinfo->GetFormatName();
    bstrPathName = qinfo->GetPathName();
    piResponse->Write(_variant_t(bstrFormatName + L"<br>"));
    piResponse->Write(_variant_t(bstrPathName + L"<p>"));
    qinfo = qinfos->Next();
  }

  return S_OK;
  
}



STDMETHODIMP Cmsgqueue::createXatQueue(BSTR bstrQueue, BSTR bstrLabel,
                                       BSTR* pbstrMessage)
{
    CComBSTR bstrMessage;
    bstrMessage = "Queue Created";
    _variant_t varXact((bool)TRUE);                     

    IMSMQQueueInfoPtr qinfo("MSMQ.MSMQQueueInfo");
    
    qinfo->PathName = _bstr_t(".\\") + _bstr_t(bstrQueue);    
    qinfo->Label = bstrLabel;
    try
    {
      qinfo->Create(&varXact);
    }
    catch (_com_error comerr)
    {
      HRESULT hr = comerr.Error();
      if (hr == MQ_ERROR_QUEUE_EXISTS)
      {
        bstrMessage = "Queue already exists";
      }
      else
      {
        bstrMessage = "Could not create queue";
      }
    }

    *pbstrMessage = bstrMessage.Detach();
 
	return S_OK;
}
