// message.cpp : Implementation of Cmessage
#include "stdafx.h"
#include "Asp1803.h"
#include "message.h"
#import "mqoa.dll" no_namespace
/////////////////////////////////////////////////////////////////////////////
// Cmessage
HRESULT Cmessage::Activate()
{
   HRESULT hr = S_OK;
   CComBSTR bstrObj;
   CComVariant vt;
   CComPtr<IGetContextProperties> pProps; //Context Properties

   IDispatch* piDispatch = NULL;

   // get ObjectContext
	hr = CoGetObjectContext(IID_IObjectContext, 
                                    (void **)&m_spObjectContext);
   if (FAILED(hr))
      return hr;
   
   // get Context Properties
   hr = m_spObjectContext->QueryInterface( IID_IGetContextProperties, 
               (void**)&pProps );
   if (FAILED(hr)) 
       return hr;

   // get Response property
   bstrObj = "Response";
   hr = pProps->GetProperty( bstrObj, &vt ) ;

   if (FAILED(hr))
       return hr;

   piDispatch = vt. pdispVal;
   hr = piDispatch->QueryInterface( IID_IResponse, 
                     (void**)&m_piResponse );

   return hr;
} 

BOOL Cmessage::CanBePooled()
{
	return TRUE;
} 

void Cmessage::Deactivate()
{
    m_piResponse.Release();
	m_spObjectContext.Release();
} 


STDMETHODIMP Cmessage::sendStringMessage(BSTR bstrQueue, BSTR bstrMessage)
{
  HRESULT hr = S_OK;
  IMSMQQueueInfoPtr qinfo("MSMQ.MSMQQueueInfo");
  IMSMQQueuePtr qQueue;
  IMSMQMessagePtr msg("MSMQ.MSMQMessage");
  
  _bstr_t bstrPath = L".\\" + _bstr_t(bstrQueue);

  // open queue, send message 
  try {
      // open queue
      qinfo->PutPathName(bstrPath);

      qQueue = qinfo->Open(MQ_SEND_ACCESS, MQ_DENY_NONE);

      // send message
      msg->Body = bstrMessage;
      msg->Send(qQueue);
  }
  catch( _com_error e) {
      m_piResponse->Write(_variant_t(e.Description()));
     
  }
  

  return hr;
}

STDMETHODIMP Cmessage::readStringMessage(BSTR bstrQueue)
{
  HRESULT hr = S_OK;
  IMSMQQueueInfoPtr qinfo("MSMQ.MSMQQueueInfo");
  IMSMQQueuePtr qQueue;
  IMSMQMessagePtr msg("MSMQ.MSMQMessage");
  _variant_t varMsgBody;

  _variant_t vtReceiveTimeout;
  vtReceiveTimeout = (long)100;
  
  _bstr_t bstrPath = L".\\" + _bstr_t(bstrQueue);

  // open queue, send message 
  try {
      // open queue
      qinfo->PutPathName(bstrPath);
      //qinfo->PutLabel(bstrLabel);
      qQueue = qinfo->Open(MQ_RECEIVE_ACCESS, MQ_DENY_RECEIVE_SHARE);

      // get messages
      msg = qQueue->Receive(&vtMissing, &vtMissing, 
                             &vtMissing, &vtReceiveTimeout);
      while (msg != NULL) {
        varMsgBody = msg->GetBody();
        m_piResponse->Write(varMsgBody);
        m_piResponse->Write(_variant_t("<br>"));
        
        msg = qQueue->Receive(&vtMissing, &vtMissing, 
                             &vtMissing, &vtReceiveTimeout);
      }

  }
  catch( _com_error e) {
      m_piResponse->Write(_variant_t(e.Description()));
     
  }

  return hr;
}

STDMETHODIMP Cmessage::sendXactMsg(BSTR bstrQueue, BSTR bstrMessage)
{
  IMSMQQueueInfoPtr qinfo("MSMQ.MSMQQueueInfo");
  IMSMQQueuePtr qQueue;
  IMSMQMessagePtr msg("MSMQ.MSMQMessage");

  _bstr_t bstrPath = L".\\" + _bstr_t(bstrQueue);

  try {
      // open queue
      qinfo->PutPathName(bstrPath);
      qQueue = qinfo->Open(MQ_SEND_ACCESS, MQ_DENY_NONE);

      // send message
      msg->Body = bstrMessage;

      _variant_t varType;
      varType.lVal = MQ_SINGLE_MESSAGE;
      varType.vt = VT_I4;

      if (m_spObjectContext->IsInTransaction()) 
          msg->Send(qQueue);
      else
          msg->Send(qQueue,&varType);

  }
  catch( _com_error e) {
      m_piResponse->Write(_variant_t(e.Description()));
  }

  return S_OK;
}

STDMETHODIMP Cmessage::readXactMsg(BSTR bstrQueue)
{
  IMSMQQueueInfoPtr qinfo("MSMQ.MSMQQueueInfo");
  IMSMQQueuePtr qQueue;
  IMSMQMessagePtr msg("MSMQ.MSMQMessage");
  _variant_t varMsgBody;

  _variant_t vtReceiveTimeout;
  vtReceiveTimeout = (long)100;
  
  _bstr_t bstrPath = L".\\" + _bstr_t(bstrQueue);

  // open queue, send message 
  try {
      // open queue
      qinfo->PutPathName(bstrPath);
      qQueue = qinfo->Open(MQ_RECEIVE_ACCESS, MQ_DENY_RECEIVE_SHARE);

      _variant_t varType;
      varType.lVal = MQ_SINGLE_MESSAGE;
      varType.vt = VT_I4;

      if (m_spObjectContext->IsInTransaction()) 
          msg = qQueue->Receive(&vtMissing,&vtMissing,
                                &vtMissing,&vtReceiveTimeout);
      else
          msg = qQueue->Receive(&varType,&vtMissing,
                                &vtMissing,&vtReceiveTimeout);
      // get messages

      while (msg != NULL) {
        varMsgBody = msg->GetBody();
        m_piResponse->Write(varMsgBody);
        m_piResponse->Write(_variant_t("<br>"));
        
      if (m_spObjectContext->IsInTransaction()) 
          msg = qQueue->Receive(&vtMissing,&vtMissing,
                                &vtMissing,&vtReceiveTimeout);
      else
          msg = qQueue->Receive(&varType,&vtMissing,
                                 &vtMissing,&vtReceiveTimeout);
     }

  }
  catch( _com_error e) {
      HRESULT hr = e.Error();
      m_piResponse->Write(_variant_t(e.Description()));
      m_spObjectContext->SetAbort();
      return hr;
  }

  m_spObjectContext->SetComplete();

  return S_OK;
}
